/**
 {{ str(this) | to_c_comment }}
 */
class {{ class_name or this.class_name }} {
        construct() {
            this.lastPrediction = -1
        }

        /**
         * Predict class from features
         */
        predict(x) {
            let predictedValue = 0;

            {% filter indent(8) %}
            {% block predict %}{% endblock %}
            {% endfilter %}

            return (this.lastPrediction = predictedValue);
        }

        {% filter indent(4) %}
        {% include 'class_map' indent content %}
        {% endfilter %}

        {% filter indent(4) %}
        {% block methods %}{% endblock %}
        {% endfilter %}
};