{% if left[i] != right[i] %}
if x[{{ features[i] }}] <= {{ thresholds[i] }}:
    {% with i = left[i] %}
    {% include 'tree' indent content %}
    {% endwith %}
else:
    {% with i = right[i] %}
    {% include 'tree' indent content %}
    {% endwith %}
{% else %}
return {{ classes[i].argmax() }}, {{ classes.flatten()[classes[i].argmax()] }}
{% endif %}